from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.finding_analysis import FindingAnalysis
    from ..models.finding_attrib import FindingAttrib
    from ..models.finding_component import FindingComponent
    from ..models.finding_vulnerability import FindingVulnerability


T = TypeVar("T", bound="Finding")


@_attrs_define
class Finding:
    """
    Attributes:
        component (Union[Unset, FindingComponent]):
        vulnerability (Union[Unset, FindingVulnerability]):
        analysis (Union[Unset, FindingAnalysis]):
        attribution (Union[Unset, FindingAttrib]):
        matrix (Union[Unset, str]):
    """

    component: Union[Unset, "FindingComponent"] = UNSET
    vulnerability: Union[Unset, "FindingVulnerability"] = UNSET
    analysis: Union[Unset, "FindingAnalysis"] = UNSET
    attribution: Union[Unset, "FindingAttrib"] = UNSET
    matrix: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        component: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.component, Unset):
            component = self.component.to_dict()

        vulnerability: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.vulnerability, Unset):
            vulnerability = self.vulnerability.to_dict()

        analysis: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.analysis, Unset):
            analysis = self.analysis.to_dict()

        attribution: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.attribution, Unset):
            attribution = self.attribution.to_dict()

        matrix = self.matrix

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if component is not UNSET:
            field_dict["component"] = component
        if vulnerability is not UNSET:
            field_dict["vulnerability"] = vulnerability
        if analysis is not UNSET:
            field_dict["analysis"] = analysis
        if attribution is not UNSET:
            field_dict["attribution"] = attribution
        if matrix is not UNSET:
            field_dict["matrix"] = matrix

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.finding_analysis import FindingAnalysis
        from ..models.finding_attrib import FindingAttrib
        from ..models.finding_component import FindingComponent
        from ..models.finding_vulnerability import FindingVulnerability

        d = dict(src_dict)
        _component = d.pop("component", UNSET)
        component: Union[Unset, FindingComponent]
        if isinstance(_component, Unset):
            component = UNSET
        else:
            component = FindingComponent.from_dict(_component)

        _vulnerability = d.pop("vulnerability", UNSET)
        vulnerability: Union[Unset, FindingVulnerability]
        if isinstance(_vulnerability, Unset):
            vulnerability = UNSET
        else:
            vulnerability = FindingVulnerability.from_dict(_vulnerability)

        _analysis = d.pop("analysis", UNSET)
        analysis: Union[Unset, FindingAnalysis]
        if isinstance(_analysis, Unset):
            analysis = UNSET
        else:
            analysis = FindingAnalysis.from_dict(_analysis)

        _attribution = d.pop("attribution", UNSET)
        attribution: Union[Unset, FindingAttrib]
        if isinstance(_attribution, Unset):
            attribution = UNSET
        else:
            attribution = FindingAttrib.from_dict(_attribution)

        matrix = d.pop("matrix", UNSET)

        finding = cls(
            component=component,
            vulnerability=vulnerability,
            analysis=analysis,
            attribution=attribution,
            matrix=matrix,
        )

        finding.additional_properties = d
        return finding

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
    Union,
)
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.api_key import ApiKey
    from ..models.ldap_user import LdapUser
    from ..models.managed_user import ManagedUser
    from ..models.mapped_ldap_group import MappedLdapGroup
    from ..models.mapped_oidc_group import MappedOidcGroup
    from ..models.oidc_user import OidcUser
    from ..models.permission import Permission


T = TypeVar("T", bound="Team")


@_attrs_define
class Team:
    """
    Attributes:
        uuid (UUID):
        name (str):
        api_keys (Union[Unset, list['ApiKey']]):
        ldap_users (Union[Unset, list['LdapUser']]):
        managed_users (Union[Unset, list['ManagedUser']]):
        oidc_users (Union[Unset, list['OidcUser']]):
        mapped_ldap_groups (Union[Unset, list['MappedLdapGroup']]):
        mapped_oidc_groups (Union[Unset, list['MappedOidcGroup']]):
        permissions (Union[Unset, list['Permission']]):
    """

    uuid: UUID
    name: str
    api_keys: Union[Unset, list["ApiKey"]] = UNSET
    ldap_users: Union[Unset, list["LdapUser"]] = UNSET
    managed_users: Union[Unset, list["ManagedUser"]] = UNSET
    oidc_users: Union[Unset, list["OidcUser"]] = UNSET
    mapped_ldap_groups: Union[Unset, list["MappedLdapGroup"]] = UNSET
    mapped_oidc_groups: Union[Unset, list["MappedOidcGroup"]] = UNSET
    permissions: Union[Unset, list["Permission"]] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        uuid = str(self.uuid)

        name = self.name

        api_keys: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.api_keys, Unset):
            api_keys = []
            for api_keys_item_data in self.api_keys:
                api_keys_item = api_keys_item_data.to_dict()
                api_keys.append(api_keys_item)

        ldap_users: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.ldap_users, Unset):
            ldap_users = []
            for ldap_users_item_data in self.ldap_users:
                ldap_users_item = ldap_users_item_data.to_dict()
                ldap_users.append(ldap_users_item)

        managed_users: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.managed_users, Unset):
            managed_users = []
            for managed_users_item_data in self.managed_users:
                managed_users_item = managed_users_item_data.to_dict()
                managed_users.append(managed_users_item)

        oidc_users: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.oidc_users, Unset):
            oidc_users = []
            for oidc_users_item_data in self.oidc_users:
                oidc_users_item = oidc_users_item_data.to_dict()
                oidc_users.append(oidc_users_item)

        mapped_ldap_groups: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.mapped_ldap_groups, Unset):
            mapped_ldap_groups = []
            for mapped_ldap_groups_item_data in self.mapped_ldap_groups:
                mapped_ldap_groups_item = mapped_ldap_groups_item_data.to_dict()
                mapped_ldap_groups.append(mapped_ldap_groups_item)

        mapped_oidc_groups: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.mapped_oidc_groups, Unset):
            mapped_oidc_groups = []
            for mapped_oidc_groups_item_data in self.mapped_oidc_groups:
                mapped_oidc_groups_item = mapped_oidc_groups_item_data.to_dict()
                mapped_oidc_groups.append(mapped_oidc_groups_item)

        permissions: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.permissions, Unset):
            permissions = []
            for permissions_item_data in self.permissions:
                permissions_item = permissions_item_data.to_dict()
                permissions.append(permissions_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "uuid": uuid,
                "name": name,
            }
        )
        if api_keys is not UNSET:
            field_dict["apiKeys"] = api_keys
        if ldap_users is not UNSET:
            field_dict["ldapUsers"] = ldap_users
        if managed_users is not UNSET:
            field_dict["managedUsers"] = managed_users
        if oidc_users is not UNSET:
            field_dict["oidcUsers"] = oidc_users
        if mapped_ldap_groups is not UNSET:
            field_dict["mappedLdapGroups"] = mapped_ldap_groups
        if mapped_oidc_groups is not UNSET:
            field_dict["mappedOidcGroups"] = mapped_oidc_groups
        if permissions is not UNSET:
            field_dict["permissions"] = permissions

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.api_key import ApiKey
        from ..models.ldap_user import LdapUser
        from ..models.managed_user import ManagedUser
        from ..models.mapped_ldap_group import MappedLdapGroup
        from ..models.mapped_oidc_group import MappedOidcGroup
        from ..models.oidc_user import OidcUser
        from ..models.permission import Permission

        d = dict(src_dict)
        uuid = UUID(d.pop("uuid"))

        name = d.pop("name")

        api_keys = []
        _api_keys = d.pop("apiKeys", UNSET)
        for api_keys_item_data in _api_keys or []:
            api_keys_item = ApiKey.from_dict(api_keys_item_data)

            api_keys.append(api_keys_item)

        ldap_users = []
        _ldap_users = d.pop("ldapUsers", UNSET)
        for ldap_users_item_data in _ldap_users or []:
            ldap_users_item = LdapUser.from_dict(ldap_users_item_data)

            ldap_users.append(ldap_users_item)

        managed_users = []
        _managed_users = d.pop("managedUsers", UNSET)
        for managed_users_item_data in _managed_users or []:
            managed_users_item = ManagedUser.from_dict(managed_users_item_data)

            managed_users.append(managed_users_item)

        oidc_users = []
        _oidc_users = d.pop("oidcUsers", UNSET)
        for oidc_users_item_data in _oidc_users or []:
            oidc_users_item = OidcUser.from_dict(oidc_users_item_data)

            oidc_users.append(oidc_users_item)

        mapped_ldap_groups = []
        _mapped_ldap_groups = d.pop("mappedLdapGroups", UNSET)
        for mapped_ldap_groups_item_data in _mapped_ldap_groups or []:
            mapped_ldap_groups_item = MappedLdapGroup.from_dict(
                mapped_ldap_groups_item_data
            )

            mapped_ldap_groups.append(mapped_ldap_groups_item)

        mapped_oidc_groups = []
        _mapped_oidc_groups = d.pop("mappedOidcGroups", UNSET)
        for mapped_oidc_groups_item_data in _mapped_oidc_groups or []:
            mapped_oidc_groups_item = MappedOidcGroup.from_dict(
                mapped_oidc_groups_item_data
            )

            mapped_oidc_groups.append(mapped_oidc_groups_item)

        permissions = []
        _permissions = d.pop("permissions", UNSET)
        for permissions_item_data in _permissions or []:
            permissions_item = Permission.from_dict(permissions_item_data)

            permissions.append(permissions_item)

        team = cls(
            uuid=uuid,
            name=name,
            api_keys=api_keys,
            ldap_users=ldap_users,
            managed_users=managed_users,
            oidc_users=oidc_users,
            mapped_ldap_groups=mapped_ldap_groups,
            mapped_oidc_groups=mapped_oidc_groups,
            permissions=permissions,
        )

        team.additional_properties = d
        return team

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ConfigPropertyPropertyType(str, Enum):
    BOOLEAN = "BOOLEAN"
    ENCRYPTEDSTRING = "ENCRYPTEDSTRING"
    INTEGER = "INTEGER"
    NUMBER = "NUMBER"
    STRING = "STRING"
    TIMESTAMP = "TIMESTAMP"
    URL = "URL"
    UUID = "UUID"

    def __str__(self) -> str:
        return str(self.value)



from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.config_property_property_type import ConfigPropertyPropertyType
from ..types import UNSET, Unset

T = TypeVar("T", bound="ConfigProperty")


@_attrs_define
class ConfigProperty:
    """
    Attributes:
        group_name (str):
        property_name (str):
        property_type (ConfigPropertyPropertyType):
        property_value (Union[Unset, str]):
        description (Union[Unset, str]):
    """

    group_name: str
    property_name: str
    property_type: ConfigPropertyPropertyType
    property_value: Union[Unset, str] = UNSET
    description: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        group_name = self.group_name

        property_name = self.property_name

        property_type = self.property_type.value

        property_value = self.property_value

        description = self.description

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "groupName": group_name,
                "propertyName": property_name,
                "propertyType": property_type,
            }
        )
        if property_value is not UNSET:
            field_dict["propertyValue"] = property_value
        if description is not UNSET:
            field_dict["description"] = description

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        group_name = d.pop("groupName")

        property_name = d.pop("propertyName")

        property_type = ConfigPropertyPropertyType(d.pop("propertyType"))

        property_value = d.pop("propertyValue", UNSET)

        description = d.pop("description", UNSET)

        config_property = cls(
            group_name=group_name,
            property_name=property_name,
            property_type=property_type,
            property_value=property_value,
            description=description,
        )

        config_property.additional_properties = d
        return config_property

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
    Union,
)
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.policy_operator import PolicyOperator
from ..models.policy_violation_state import PolicyViolationState
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.policy_condition import PolicyCondition
    from ..models.project import Project
    from ..models.tag import Tag


T = TypeVar("T", bound="Policy")


@_attrs_define
class Policy:
    """
    Attributes:
        name (str):
        operator (PolicyOperator):
        violation_state (PolicyViolationState):
        uuid (UUID):
        policy_conditions (Union[Unset, list['PolicyCondition']]):
        projects (Union[Unset, list['Project']]):
        tags (Union[Unset, list['Tag']]):
        include_children (Union[Unset, bool]):
        only_latest_project_version (Union[Unset, bool]):
        global_ (Union[Unset, bool]):
    """

    name: str
    operator: PolicyOperator
    violation_state: PolicyViolationState
    uuid: UUID
    policy_conditions: Union[Unset, list["PolicyCondition"]] = UNSET
    projects: Union[Unset, list["Project"]] = UNSET
    tags: Union[Unset, list["Tag"]] = UNSET
    include_children: Union[Unset, bool] = UNSET
    only_latest_project_version: Union[Unset, bool] = UNSET
    global_: Union[Unset, bool] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        name = self.name

        operator = self.operator.value

        violation_state = self.violation_state.value

        uuid = str(self.uuid)

        policy_conditions: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.policy_conditions, Unset):
            policy_conditions = []
            for policy_conditions_item_data in self.policy_conditions:
                policy_conditions_item = policy_conditions_item_data.to_dict()
                policy_conditions.append(policy_conditions_item)

        projects: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.projects, Unset):
            projects = []
            for projects_item_data in self.projects:
                projects_item = projects_item_data.to_dict()
                projects.append(projects_item)

        tags: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.tags, Unset):
            tags = []
            for tags_item_data in self.tags:
                tags_item = tags_item_data.to_dict()
                tags.append(tags_item)

        include_children = self.include_children

        only_latest_project_version = self.only_latest_project_version

        global_ = self.global_

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "operator": operator,
                "violationState": violation_state,
                "uuid": uuid,
            }
        )
        if policy_conditions is not UNSET:
            field_dict["policyConditions"] = policy_conditions
        if projects is not UNSET:
            field_dict["projects"] = projects
        if tags is not UNSET:
            field_dict["tags"] = tags
        if include_children is not UNSET:
            field_dict["includeChildren"] = include_children
        if only_latest_project_version is not UNSET:
            field_dict["onlyLatestProjectVersion"] = only_latest_project_version
        if global_ is not UNSET:
            field_dict["global"] = global_

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.policy_condition import PolicyCondition
        from ..models.project import Project
        from ..models.tag import Tag

        d = dict(src_dict)
        name = d.pop("name")

        operator = PolicyOperator(d.pop("operator"))

        violation_state = PolicyViolationState(d.pop("violationState"))

        uuid = UUID(d.pop("uuid"))

        policy_conditions = []
        _policy_conditions = d.pop("policyConditions", UNSET)
        for policy_conditions_item_data in _policy_conditions or []:
            policy_conditions_item = PolicyCondition.from_dict(
                policy_conditions_item_data
            )

            policy_conditions.append(policy_conditions_item)

        projects = []
        _projects = d.pop("projects", UNSET)
        for projects_item_data in _projects or []:
            projects_item = Project.from_dict(projects_item_data)

            projects.append(projects_item)

        tags = []
        _tags = d.pop("tags", UNSET)
        for tags_item_data in _tags or []:
            tags_item = Tag.from_dict(tags_item_data)

            tags.append(tags_item)

        include_children = d.pop("includeChildren", UNSET)

        only_latest_project_version = d.pop("onlyLatestProjectVersion", UNSET)

        global_ = d.pop("global", UNSET)

        policy = cls(
            name=name,
            operator=operator,
            violation_state=violation_state,
            uuid=uuid,
            policy_conditions=policy_conditions,
            projects=projects,
            tags=tags,
            include_children=include_children,
            only_latest_project_version=only_latest_project_version,
            global_=global_,
        )

        policy.additional_properties = d
        return policy

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
    Union,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.organizational_contact import OrganizationalContact


T = TypeVar("T", bound="OrganizationalEntity")


@_attrs_define
class OrganizationalEntity:
    """
    Attributes:
        name (Union[Unset, str]):
        urls (Union[Unset, list[str]]):
        contacts (Union[Unset, list['OrganizationalContact']]):
    """

    name: Union[Unset, str] = UNSET
    urls: Union[Unset, list[str]] = UNSET
    contacts: Union[Unset, list["OrganizationalContact"]] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        name = self.name

        urls: Union[Unset, list[str]] = UNSET
        if not isinstance(self.urls, Unset):
            urls = self.urls

        contacts: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.contacts, Unset):
            contacts = []
            for contacts_item_data in self.contacts:
                contacts_item = contacts_item_data.to_dict()
                contacts.append(contacts_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if name is not UNSET:
            field_dict["name"] = name
        if urls is not UNSET:
            field_dict["urls"] = urls
        if contacts is not UNSET:
            field_dict["contacts"] = contacts

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.organizational_contact import OrganizationalContact

        d = dict(src_dict)
        name = d.pop("name", UNSET)

        urls = cast(list[str], d.pop("urls", UNSET))

        contacts = []
        _contacts = d.pop("contacts", UNSET)
        for contacts_item_data in _contacts or []:
            contacts_item = OrganizationalContact.from_dict(contacts_item_data)

            contacts.append(contacts_item)

        organizational_entity = cls(
            name=name,
            urls=urls,
            contacts=contacts,
        )

        organizational_entity.additional_properties = d
        return organizational_entity

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Optional, Union, cast
from uuid import UUID

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.component import Component
from ...models.get_all_components_sort_order import GetAllComponentsSortOrder
from ...types import UNSET, Response, Unset


def _get_kwargs(
    uuid: UUID,
    *,
    page_number: Union[Unset, str] = "1",
    page_size: Union[Unset, str] = "100",
    offset: Union[Unset, str] = UNSET,
    limit: Union[Unset, str] = UNSET,
    sort_name: Union[Unset, str] = UNSET,
    sort_order: Union[Unset, GetAllComponentsSortOrder] = UNSET,
    only_outdated: Union[Unset, bool] = UNSET,
    only_direct: Union[Unset, bool] = UNSET,
) -> dict[str, Any]:
    params: dict[str, Any] = {}

    params["pageNumber"] = page_number

    params["pageSize"] = page_size

    params["offset"] = offset

    params["limit"] = limit

    params["sortName"] = sort_name

    json_sort_order: Union[Unset, str] = UNSET
    if not isinstance(sort_order, Unset):
        json_sort_order = sort_order.value

    params["sortOrder"] = json_sort_order

    params["onlyOutdated"] = only_outdated

    params["onlyDirect"] = only_direct

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/v1/component/project/{uuid}".format(
            uuid=uuid,
        ),
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[Any, list["Component"]]]:
    if response.status_code == 200:
        response_200 = []
        _response_200 = response.json()
        for response_200_item_data in _response_200:
            response_200_item = Component.from_dict(response_200_item_data)

            response_200.append(response_200_item)

        return response_200

    if response.status_code == 401:
        response_401 = cast(Any, None)
        return response_401

    if response.status_code == 403:
        response_403 = cast(Any, None)
        return response_403

    if response.status_code == 404:
        response_404 = cast(Any, None)
        return response_404

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[Any, list["Component"]]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    uuid: UUID,
    *,
    client: AuthenticatedClient,
    page_number: Union[Unset, str] = "1",
    page_size: Union[Unset, str] = "100",
    offset: Union[Unset, str] = UNSET,
    limit: Union[Unset, str] = UNSET,
    sort_name: Union[Unset, str] = UNSET,
    sort_order: Union[Unset, GetAllComponentsSortOrder] = UNSET,
    only_outdated: Union[Unset, bool] = UNSET,
    only_direct: Union[Unset, bool] = UNSET,
) -> Response[Union[Any, list["Component"]]]:
    """Returns a list of all components for a given project

     <p>Requires permission <strong>VIEW_PORTFOLIO</strong></p>

    Args:
        uuid (UUID):
        page_number (Union[Unset, str]):  Default: '1'.
        page_size (Union[Unset, str]):  Default: '100'.
        offset (Union[Unset, str]):
        limit (Union[Unset, str]):
        sort_name (Union[Unset, str]):
        sort_order (Union[Unset, GetAllComponentsSortOrder]):
        only_outdated (Union[Unset, bool]):
        only_direct (Union[Unset, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, list['Component']]]
    """

    kwargs = _get_kwargs(
        uuid=uuid,
        page_number=page_number,
        page_size=page_size,
        offset=offset,
        limit=limit,
        sort_name=sort_name,
        sort_order=sort_order,
        only_outdated=only_outdated,
        only_direct=only_direct,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    uuid: UUID,
    *,
    client: AuthenticatedClient,
    page_number: Union[Unset, str] = "1",
    page_size: Union[Unset, str] = "100",
    offset: Union[Unset, str] = UNSET,
    limit: Union[Unset, str] = UNSET,
    sort_name: Union[Unset, str] = UNSET,
    sort_order: Union[Unset, GetAllComponentsSortOrder] = UNSET,
    only_outdated: Union[Unset, bool] = UNSET,
    only_direct: Union[Unset, bool] = UNSET,
) -> Optional[Union[Any, list["Component"]]]:
    """Returns a list of all components for a given project

     <p>Requires permission <strong>VIEW_PORTFOLIO</strong></p>

    Args:
        uuid (UUID):
        page_number (Union[Unset, str]):  Default: '1'.
        page_size (Union[Unset, str]):  Default: '100'.
        offset (Union[Unset, str]):
        limit (Union[Unset, str]):
        sort_name (Union[Unset, str]):
        sort_order (Union[Unset, GetAllComponentsSortOrder]):
        only_outdated (Union[Unset, bool]):
        only_direct (Union[Unset, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, list['Component']]
    """

    return sync_detailed(
        uuid=uuid,
        client=client,
        page_number=page_number,
        page_size=page_size,
        offset=offset,
        limit=limit,
        sort_name=sort_name,
        sort_order=sort_order,
        only_outdated=only_outdated,
        only_direct=only_direct,
    ).parsed


async def asyncio_detailed(
    uuid: UUID,
    *,
    client: AuthenticatedClient,
    page_number: Union[Unset, str] = "1",
    page_size: Union[Unset, str] = "100",
    offset: Union[Unset, str] = UNSET,
    limit: Union[Unset, str] = UNSET,
    sort_name: Union[Unset, str] = UNSET,
    sort_order: Union[Unset, GetAllComponentsSortOrder] = UNSET,
    only_outdated: Union[Unset, bool] = UNSET,
    only_direct: Union[Unset, bool] = UNSET,
) -> Response[Union[Any, list["Component"]]]:
    """Returns a list of all components for a given project

     <p>Requires permission <strong>VIEW_PORTFOLIO</strong></p>

    Args:
        uuid (UUID):
        page_number (Union[Unset, str]):  Default: '1'.
        page_size (Union[Unset, str]):  Default: '100'.
        offset (Union[Unset, str]):
        limit (Union[Unset, str]):
        sort_name (Union[Unset, str]):
        sort_order (Union[Unset, GetAllComponentsSortOrder]):
        only_outdated (Union[Unset, bool]):
        only_direct (Union[Unset, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, list['Component']]]
    """

    kwargs = _get_kwargs(
        uuid=uuid,
        page_number=page_number,
        page_size=page_size,
        offset=offset,
        limit=limit,
        sort_name=sort_name,
        sort_order=sort_order,
        only_outdated=only_outdated,
        only_direct=only_direct,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    uuid: UUID,
    *,
    client: AuthenticatedClient,
    page_number: Union[Unset, str] = "1",
    page_size: Union[Unset, str] = "100",
    offset: Union[Unset, str] = UNSET,
    limit: Union[Unset, str] = UNSET,
    sort_name: Union[Unset, str] = UNSET,
    sort_order: Union[Unset, GetAllComponentsSortOrder] = UNSET,
    only_outdated: Union[Unset, bool] = UNSET,
    only_direct: Union[Unset, bool] = UNSET,
) -> Optional[Union[Any, list["Component"]]]:
    """Returns a list of all components for a given project

     <p>Requires permission <strong>VIEW_PORTFOLIO</strong></p>

    Args:
        uuid (UUID):
        page_number (Union[Unset, str]):  Default: '1'.
        page_size (Union[Unset, str]):  Default: '100'.
        offset (Union[Unset, str]):
        limit (Union[Unset, str]):
        sort_name (Union[Unset, str]):
        sort_order (Union[Unset, GetAllComponentsSortOrder]):
        only_outdated (Union[Unset, bool]):
        only_direct (Union[Unset, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, list['Component']]
    """

    return (
        await asyncio_detailed(
            uuid=uuid,
            client=client,
            page_number=page_number,
            page_size=page_size,
            offset=offset,
            limit=limit,
            sort_name=sort_name,
            sort_order=sort_order,
            only_outdated=only_outdated,
            only_direct=only_direct,
        )
    ).parsed



from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
    Union,
)
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.affected_component_identity_type import AffectedComponentIdentityType
from ..models.affected_component_version_type import AffectedComponentVersionType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.affected_version_attribution import AffectedVersionAttribution


T = TypeVar("T", bound="AffectedComponent")


@_attrs_define
class AffectedComponent:
    """
    Attributes:
        identity_type (Union[Unset, AffectedComponentIdentityType]):
        identity (Union[Unset, str]):
        version_type (Union[Unset, AffectedComponentVersionType]):
        version (Union[Unset, str]):
        version_end_excluding (Union[Unset, str]):
        version_end_including (Union[Unset, str]):
        version_start_excluding (Union[Unset, str]):
        version_start_including (Union[Unset, str]):
        uuid (Union[Unset, UUID]):
        affected_version_attributions (Union[Unset, list['AffectedVersionAttribution']]):
    """

    identity_type: Union[Unset, AffectedComponentIdentityType] = UNSET
    identity: Union[Unset, str] = UNSET
    version_type: Union[Unset, AffectedComponentVersionType] = UNSET
    version: Union[Unset, str] = UNSET
    version_end_excluding: Union[Unset, str] = UNSET
    version_end_including: Union[Unset, str] = UNSET
    version_start_excluding: Union[Unset, str] = UNSET
    version_start_including: Union[Unset, str] = UNSET
    uuid: Union[Unset, UUID] = UNSET
    affected_version_attributions: Union[Unset, list["AffectedVersionAttribution"]] = (
        UNSET
    )
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        identity_type: Union[Unset, str] = UNSET
        if not isinstance(self.identity_type, Unset):
            identity_type = self.identity_type.value

        identity = self.identity

        version_type: Union[Unset, str] = UNSET
        if not isinstance(self.version_type, Unset):
            version_type = self.version_type.value

        version = self.version

        version_end_excluding = self.version_end_excluding

        version_end_including = self.version_end_including

        version_start_excluding = self.version_start_excluding

        version_start_including = self.version_start_including

        uuid: Union[Unset, str] = UNSET
        if not isinstance(self.uuid, Unset):
            uuid = str(self.uuid)

        affected_version_attributions: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.affected_version_attributions, Unset):
            affected_version_attributions = []
            for (
                affected_version_attributions_item_data
            ) in self.affected_version_attributions:
                affected_version_attributions_item = (
                    affected_version_attributions_item_data.to_dict()
                )
                affected_version_attributions.append(affected_version_attributions_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if identity_type is not UNSET:
            field_dict["identityType"] = identity_type
        if identity is not UNSET:
            field_dict["identity"] = identity
        if version_type is not UNSET:
            field_dict["versionType"] = version_type
        if version is not UNSET:
            field_dict["version"] = version
        if version_end_excluding is not UNSET:
            field_dict["versionEndExcluding"] = version_end_excluding
        if version_end_including is not UNSET:
            field_dict["versionEndIncluding"] = version_end_including
        if version_start_excluding is not UNSET:
            field_dict["versionStartExcluding"] = version_start_excluding
        if version_start_including is not UNSET:
            field_dict["versionStartIncluding"] = version_start_including
        if uuid is not UNSET:
            field_dict["uuid"] = uuid
        if affected_version_attributions is not UNSET:
            field_dict["affectedVersionAttributions"] = affected_version_attributions

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.affected_version_attribution import AffectedVersionAttribution

        d = dict(src_dict)
        _identity_type = d.pop("identityType", UNSET)
        identity_type: Union[Unset, AffectedComponentIdentityType]
        if isinstance(_identity_type, Unset):
            identity_type = UNSET
        else:
            identity_type = AffectedComponentIdentityType(_identity_type)

        identity = d.pop("identity", UNSET)

        _version_type = d.pop("versionType", UNSET)
        version_type: Union[Unset, AffectedComponentVersionType]
        if isinstance(_version_type, Unset):
            version_type = UNSET
        else:
            version_type = AffectedComponentVersionType(_version_type)

        version = d.pop("version", UNSET)

        version_end_excluding = d.pop("versionEndExcluding", UNSET)

        version_end_including = d.pop("versionEndIncluding", UNSET)

        version_start_excluding = d.pop("versionStartExcluding", UNSET)

        version_start_including = d.pop("versionStartIncluding", UNSET)

        _uuid = d.pop("uuid", UNSET)
        uuid: Union[Unset, UUID]
        if isinstance(_uuid, Unset):
            uuid = UNSET
        else:
            uuid = UUID(_uuid)

        affected_version_attributions = []
        _affected_version_attributions = d.pop("affectedVersionAttributions", UNSET)
        for affected_version_attributions_item_data in (
            _affected_version_attributions or []
        ):
            affected_version_attributions_item = AffectedVersionAttribution.from_dict(
                affected_version_attributions_item_data
            )

            affected_version_attributions.append(affected_version_attributions_item)

        affected_component = cls(
            identity_type=identity_type,
            identity=identity,
            version_type=version_type,
            version=version,
            version_end_excluding=version_end_excluding,
            version_end_including=version_end_including,
            version_start_excluding=version_start_excluding,
            version_start_including=version_start_including,
            uuid=uuid,
            affected_version_attributions=affected_version_attributions,
        )

        affected_component.additional_properties = d
        return affected_component

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
    Union,
)
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.license_ import License


T = TypeVar("T", bound="LicenseGroup")


@_attrs_define
class LicenseGroup:
    """
    Attributes:
        name (str):
        uuid (UUID):
        licenses (Union[Unset, list['License']]):
        risk_weight (Union[Unset, int]):
    """

    name: str
    uuid: UUID
    licenses: Union[Unset, list["License"]] = UNSET
    risk_weight: Union[Unset, int] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        name = self.name

        uuid = str(self.uuid)

        licenses: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.licenses, Unset):
            licenses = []
            for licenses_item_data in self.licenses:
                licenses_item = licenses_item_data.to_dict()
                licenses.append(licenses_item)

        risk_weight = self.risk_weight

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "uuid": uuid,
            }
        )
        if licenses is not UNSET:
            field_dict["licenses"] = licenses
        if risk_weight is not UNSET:
            field_dict["riskWeight"] = risk_weight

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.license_ import License

        d = dict(src_dict)
        name = d.pop("name")

        uuid = UUID(d.pop("uuid"))

        licenses = []
        _licenses = d.pop("licenses", UNSET)
        for licenses_item_data in _licenses or []:
            licenses_item = License.from_dict(licenses_item_data)

            licenses.append(licenses_item)

        risk_weight = d.pop("riskWeight", UNSET)

        license_group = cls(
            name=name,
            uuid=uuid,
            licenses=licenses,
            risk_weight=risk_weight,
        )

        license_group.additional_properties = d
        return license_group

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.about_provider_data import AboutProviderData
    from ..models.framework import Framework


T = TypeVar("T", bound="About")


@_attrs_define
class About:
    """
    Attributes:
        version (Union[Unset, str]):
        timestamp (Union[Unset, str]):
        system_uuid (Union[Unset, str]):
        uuid (Union[Unset, str]):
        application (Union[Unset, str]):
        framework (Union[Unset, Framework]):
        provider_data (Union[Unset, AboutProviderData]):
    """

    version: Union[Unset, str] = UNSET
    timestamp: Union[Unset, str] = UNSET
    system_uuid: Union[Unset, str] = UNSET
    uuid: Union[Unset, str] = UNSET
    application: Union[Unset, str] = UNSET
    framework: Union[Unset, "Framework"] = UNSET
    provider_data: Union[Unset, "AboutProviderData"] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        version = self.version

        timestamp = self.timestamp

        system_uuid = self.system_uuid

        uuid = self.uuid

        application = self.application

        framework: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.framework, Unset):
            framework = self.framework.to_dict()

        provider_data: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.provider_data, Unset):
            provider_data = self.provider_data.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if version is not UNSET:
            field_dict["version"] = version
        if timestamp is not UNSET:
            field_dict["timestamp"] = timestamp
        if system_uuid is not UNSET:
            field_dict["systemUuid"] = system_uuid
        if uuid is not UNSET:
            field_dict["uuid"] = uuid
        if application is not UNSET:
            field_dict["application"] = application
        if framework is not UNSET:
            field_dict["framework"] = framework
        if provider_data is not UNSET:
            field_dict["providerData"] = provider_data

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.about_provider_data import AboutProviderData
        from ..models.framework import Framework

        d = dict(src_dict)
        version = d.pop("version", UNSET)

        timestamp = d.pop("timestamp", UNSET)

        system_uuid = d.pop("systemUuid", UNSET)

        uuid = d.pop("uuid", UNSET)

        application = d.pop("application", UNSET)

        _framework = d.pop("framework", UNSET)
        framework: Union[Unset, Framework]
        if isinstance(_framework, Unset):
            framework = UNSET
        else:
            framework = Framework.from_dict(_framework)

        _provider_data = d.pop("providerData", UNSET)
        provider_data: Union[Unset, AboutProviderData]
        if isinstance(_provider_data, Unset):
            provider_data = UNSET
        else:
            provider_data = AboutProviderData.from_dict(_provider_data)

        about = cls(
            version=version,
            timestamp=timestamp,
            system_uuid=system_uuid,
            uuid=uuid,
            application=application,
            framework=framework,
            provider_data=provider_data,
        )

        about.additional_properties = d
        return about

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class PolicyViolationType(str, Enum):
    LICENSE = "LICENSE"
    OPERATIONAL = "OPERATIONAL"
    SECURITY = "SECURITY"

    def __str__(self) -> str:
        return str(self.value)



from http import HTTPStatus
from typing import Any, Optional, Union, cast
from uuid import UUID

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.is_token_being_processed_response import IsTokenBeingProcessedResponse
from ...types import Response


def _get_kwargs(
    uuid: UUID,
) -> dict[str, Any]:
    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/v1/event/token/{uuid}".format(
            uuid=uuid,
        ),
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[Any, IsTokenBeingProcessedResponse]]:
    if response.status_code == 200:
        response_200 = IsTokenBeingProcessedResponse.from_dict(response.json())

        return response_200

    if response.status_code == 401:
        response_401 = cast(Any, None)
        return response_401

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[Any, IsTokenBeingProcessedResponse]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    uuid: UUID,
    *,
    client: AuthenticatedClient,
) -> Response[Union[Any, IsTokenBeingProcessedResponse]]:
    """Determines if there are any tasks associated with the token that are being processed, or in the
    queue to be processed.

     <p>
      This endpoint is intended to be used in conjunction with other API calls which return a token for
    asynchronous tasks.
      The token can then be queried using this endpoint to determine if the task is complete:
      <ul>
        <li>A value of <code>true</code> indicates processing is occurring.</li>
        <li>A value of <code>false</code> indicates that no processing is occurring for the specified
    token.</li>
      </ul>
      However, a value of <code>false</code> also does not confirm the token is valid,
      only that no processing is associated with the specified token.
    </p>

    Args:
        uuid (UUID):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, IsTokenBeingProcessedResponse]]
    """

    kwargs = _get_kwargs(
        uuid=uuid,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    uuid: UUID,
    *,
    client: AuthenticatedClient,
) -> Optional[Union[Any, IsTokenBeingProcessedResponse]]:
    """Determines if there are any tasks associated with the token that are being processed, or in the
    queue to be processed.

     <p>
      This endpoint is intended to be used in conjunction with other API calls which return a token for
    asynchronous tasks.
      The token can then be queried using this endpoint to determine if the task is complete:
      <ul>
        <li>A value of <code>true</code> indicates processing is occurring.</li>
        <li>A value of <code>false</code> indicates that no processing is occurring for the specified
    token.</li>
      </ul>
      However, a value of <code>false</code> also does not confirm the token is valid,
      only that no processing is associated with the specified token.
    </p>

    Args:
        uuid (UUID):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, IsTokenBeingProcessedResponse]
    """

    return sync_detailed(
        uuid=uuid,
        client=client,
    ).parsed


async def asyncio_detailed(
    uuid: UUID,
    *,
    client: AuthenticatedClient,
) -> Response[Union[Any, IsTokenBeingProcessedResponse]]:
    """Determines if there are any tasks associated with the token that are being processed, or in the
    queue to be processed.

     <p>
      This endpoint is intended to be used in conjunction with other API calls which return a token for
    asynchronous tasks.
      The token can then be queried using this endpoint to determine if the task is complete:
      <ul>
        <li>A value of <code>true</code> indicates processing is occurring.</li>
        <li>A value of <code>false</code> indicates that no processing is occurring for the specified
    token.</li>
      </ul>
      However, a value of <code>false</code> also does not confirm the token is valid,
      only that no processing is associated with the specified token.
    </p>

    Args:
        uuid (UUID):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, IsTokenBeingProcessedResponse]]
    """

    kwargs = _get_kwargs(
        uuid=uuid,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    uuid: UUID,
    *,
    client: AuthenticatedClient,
) -> Optional[Union[Any, IsTokenBeingProcessedResponse]]:
    """Determines if there are any tasks associated with the token that are being processed, or in the
    queue to be processed.

     <p>
      This endpoint is intended to be used in conjunction with other API calls which return a token for
    asynchronous tasks.
      The token can then be queried using this endpoint to determine if the task is complete:
      <ul>
        <li>A value of <code>true</code> indicates processing is occurring.</li>
        <li>A value of <code>false</code> indicates that no processing is occurring for the specified
    token.</li>
      </ul>
      However, a value of <code>false</code> also does not confirm the token is valid,
      only that no processing is associated with the specified token.
    </p>

    Args:
        uuid (UUID):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, IsTokenBeingProcessedResponse]
    """

    return (
        await asyncio_detailed(
            uuid=uuid,
            client=client,
        )
    ).parsed



from http import HTTPStatus
from typing import Any, Optional, Union
from uuid import UUID

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...types import Response


def _get_kwargs(
    uuid: UUID,
) -> dict[str, Any]:
    _kwargs: dict[str, Any] = {
        "method": "post",
        "url": "/v1/notification/publisher/test/{uuid}".format(
            uuid=uuid,
        ),
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Any]:
    if response.status_code == 200:
        return None

    if response.status_code == 401:
        return None

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Any]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    uuid: UUID,
    *,
    client: AuthenticatedClient,
) -> Response[Any]:
    """Dispatches a rule notification test

     <p>Requires permission <strong>SYSTEM_CONFIGURATION</strong></p>

    Args:
        uuid (UUID):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        uuid=uuid,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


async def asyncio_detailed(
    uuid: UUID,
    *,
    client: AuthenticatedClient,
) -> Response[Any]:
    """Dispatches a rule notification test

     <p>Requires permission <strong>SYSTEM_CONFIGURATION</strong></p>

    Args:
        uuid (UUID):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        uuid=uuid,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)



from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
    Union,
)
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.permission import Permission


T = TypeVar("T", bound="TeamSelfResponse")


@_attrs_define
class TeamSelfResponse:
    """
    Attributes:
        uuid (Union[Unset, UUID]):
        name (Union[Unset, str]):
        permissions (Union[Unset, list['Permission']]):
    """

    uuid: Union[Unset, UUID] = UNSET
    name: Union[Unset, str] = UNSET
    permissions: Union[Unset, list["Permission"]] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        uuid: Union[Unset, str] = UNSET
        if not isinstance(self.uuid, Unset):
            uuid = str(self.uuid)

        name = self.name

        permissions: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.permissions, Unset):
            permissions = []
            for permissions_item_data in self.permissions:
                permissions_item = permissions_item_data.to_dict()
                permissions.append(permissions_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if uuid is not UNSET:
            field_dict["uuid"] = uuid
        if name is not UNSET:
            field_dict["name"] = name
        if permissions is not UNSET:
            field_dict["permissions"] = permissions

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.permission import Permission

        d = dict(src_dict)
        _uuid = d.pop("uuid", UNSET)
        uuid: Union[Unset, UUID]
        if isinstance(_uuid, Unset):
            uuid = UNSET
        else:
            uuid = UUID(_uuid)

        name = d.pop("name", UNSET)

        permissions = []
        _permissions = d.pop("permissions", UNSET)
        for permissions_item_data in _permissions or []:
            permissions_item = Permission.from_dict(permissions_item_data)

            permissions.append(permissions_item)

        team_self_response = cls(
            uuid=uuid,
            name=name,
            permissions=permissions,
        )

        team_self_response.additional_properties = d
        return team_self_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import pytest
from tinystream import Stream

import owasp_dt
from owasp_dt.api.team import get_teams, create_team, delete_team
from owasp_dt.models import Team

test_team = Team(
    uuid="",
    name="test-team",
)

def test_create_team(client: owasp_dt.Client):
    global test_team
    resp = create_team.sync_detailed(client=client, body=test_team)
    assert resp.status_code == 201
    test_team = resp.parsed

@pytest.mark.depends(on=["test_create_team"])
def test_created_team(client: owasp_dt.Client):
    teams = get_teams.sync(client=client)
    assert Stream(teams).filter(lambda team: team.name == "test-team").count() == 1


@pytest.mark.depends(on=["test_created_team"])
def test_delete_team(client: owasp_dt.Client):
    resp = delete_team.sync_detailed(client=client, body=test_team)
    assert resp.status_code == 204

@pytest.mark.depends(on=["test_delete_team"])
def test_deleted_team(client: owasp_dt.Client):
    teams = get_teams.sync(client=client)
    assert Stream(teams).filter(lambda team: team.name == "test-team").count() == 0



from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.score_business_impact import ScoreBusinessImpact
from ..models.score_likelihood import ScoreLikelihood
from ..models.score_technical_impact import ScoreTechnicalImpact
from ..types import UNSET, Unset

T = TypeVar("T", bound="Score")


@_attrs_define
class Score:
    """
    Attributes:
        likelihood_score (Union[Unset, float]):
        technical_impact_score (Union[Unset, float]):
        business_impact_score (Union[Unset, float]):
        likelihood (Union[Unset, ScoreLikelihood]):
        technical_impact (Union[Unset, ScoreTechnicalImpact]):
        business_impact (Union[Unset, ScoreBusinessImpact]):
    """

    likelihood_score: Union[Unset, float] = UNSET
    technical_impact_score: Union[Unset, float] = UNSET
    business_impact_score: Union[Unset, float] = UNSET
    likelihood: Union[Unset, ScoreLikelihood] = UNSET
    technical_impact: Union[Unset, ScoreTechnicalImpact] = UNSET
    business_impact: Union[Unset, ScoreBusinessImpact] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        likelihood_score = self.likelihood_score

        technical_impact_score = self.technical_impact_score

        business_impact_score = self.business_impact_score

        likelihood: Union[Unset, str] = UNSET
        if not isinstance(self.likelihood, Unset):
            likelihood = self.likelihood.value

        technical_impact: Union[Unset, str] = UNSET
        if not isinstance(self.technical_impact, Unset):
            technical_impact = self.technical_impact.value

        business_impact: Union[Unset, str] = UNSET
        if not isinstance(self.business_impact, Unset):
            business_impact = self.business_impact.value

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if likelihood_score is not UNSET:
            field_dict["likelihoodScore"] = likelihood_score
        if technical_impact_score is not UNSET:
            field_dict["technicalImpactScore"] = technical_impact_score
        if business_impact_score is not UNSET:
            field_dict["businessImpactScore"] = business_impact_score
        if likelihood is not UNSET:
            field_dict["likelihood"] = likelihood
        if technical_impact is not UNSET:
            field_dict["technicalImpact"] = technical_impact
        if business_impact is not UNSET:
            field_dict["businessImpact"] = business_impact

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        likelihood_score = d.pop("likelihoodScore", UNSET)

        technical_impact_score = d.pop("technicalImpactScore", UNSET)

        business_impact_score = d.pop("businessImpactScore", UNSET)

        _likelihood = d.pop("likelihood", UNSET)
        likelihood: Union[Unset, ScoreLikelihood]
        if isinstance(_likelihood, Unset):
            likelihood = UNSET
        else:
            likelihood = ScoreLikelihood(_likelihood)

        _technical_impact = d.pop("technicalImpact", UNSET)
        technical_impact: Union[Unset, ScoreTechnicalImpact]
        if isinstance(_technical_impact, Unset):
            technical_impact = UNSET
        else:
            technical_impact = ScoreTechnicalImpact(_technical_impact)

        _business_impact = d.pop("businessImpact", UNSET)
        business_impact: Union[Unset, ScoreBusinessImpact]
        if isinstance(_business_impact, Unset):
            business_impact = UNSET
        else:
            business_impact = ScoreBusinessImpact(_business_impact)

        score = cls(
            likelihood_score=likelihood_score,
            technical_impact_score=technical_impact_score,
            business_impact_score=business_impact_score,
            likelihood=likelihood,
            technical_impact=technical_impact,
            business_impact=business_impact,
        )

        score.additional_properties = d
        return score

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="CloneProjectRequest")


@_attrs_define
class CloneProjectRequest:
    """
    Attributes:
        project (str):
        version (str):
        include_tags (Union[Unset, bool]):
        include_properties (Union[Unset, bool]):
        include_dependencies (Union[Unset, bool]):
        include_components (Union[Unset, bool]):
        include_services (Union[Unset, bool]):
        include_audit_history (Union[Unset, bool]):
        include_acl (Union[Unset, bool]):
        include_policy_violations (Union[Unset, bool]):
        make_clone_latest (Union[Unset, bool]):
    """

    project: str
    version: str
    include_tags: Union[Unset, bool] = UNSET
    include_properties: Union[Unset, bool] = UNSET
    include_dependencies: Union[Unset, bool] = UNSET
    include_components: Union[Unset, bool] = UNSET
    include_services: Union[Unset, bool] = UNSET
    include_audit_history: Union[Unset, bool] = UNSET
    include_acl: Union[Unset, bool] = UNSET
    include_policy_violations: Union[Unset, bool] = UNSET
    make_clone_latest: Union[Unset, bool] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        project = self.project

        version = self.version

        include_tags = self.include_tags

        include_properties = self.include_properties

        include_dependencies = self.include_dependencies

        include_components = self.include_components

        include_services = self.include_services

        include_audit_history = self.include_audit_history

        include_acl = self.include_acl

        include_policy_violations = self.include_policy_violations

        make_clone_latest = self.make_clone_latest

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "project": project,
                "version": version,
            }
        )
        if include_tags is not UNSET:
            field_dict["includeTags"] = include_tags
        if include_properties is not UNSET:
            field_dict["includeProperties"] = include_properties
        if include_dependencies is not UNSET:
            field_dict["includeDependencies"] = include_dependencies
        if include_components is not UNSET:
            field_dict["includeComponents"] = include_components
        if include_services is not UNSET:
            field_dict["includeServices"] = include_services
        if include_audit_history is not UNSET:
            field_dict["includeAuditHistory"] = include_audit_history
        if include_acl is not UNSET:
            field_dict["includeACL"] = include_acl
        if include_policy_violations is not UNSET:
            field_dict["includePolicyViolations"] = include_policy_violations
        if make_clone_latest is not UNSET:
            field_dict["makeCloneLatest"] = make_clone_latest

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        project = d.pop("project")

        version = d.pop("version")

        include_tags = d.pop("includeTags", UNSET)

        include_properties = d.pop("includeProperties", UNSET)

        include_dependencies = d.pop("includeDependencies", UNSET)

        include_components = d.pop("includeComponents", UNSET)

        include_services = d.pop("includeServices", UNSET)

        include_audit_history = d.pop("includeAuditHistory", UNSET)

        include_acl = d.pop("includeACL", UNSET)

        include_policy_violations = d.pop("includePolicyViolations", UNSET)

        make_clone_latest = d.pop("makeCloneLatest", UNSET)

        clone_project_request = cls(
            project=project,
            version=version,
            include_tags=include_tags,
            include_properties=include_properties,
            include_dependencies=include_dependencies,
            include_components=include_components,
            include_services=include_services,
            include_audit_history=include_audit_history,
            include_acl=include_acl,
            include_policy_violations=include_policy_violations,
            make_clone_latest=make_clone_latest,
        )

        clone_project_request.additional_properties = d
        return clone_project_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ProjectMetricsCollectionLogic(str, Enum):
    AGGREGATE_DIRECT_CHILDREN = "AGGREGATE_DIRECT_CHILDREN"
    AGGREGATE_DIRECT_CHILDREN_WITH_TAG = "AGGREGATE_DIRECT_CHILDREN_WITH_TAG"
    AGGREGATE_LATEST_VERSION_CHILDREN = "AGGREGATE_LATEST_VERSION_CHILDREN"
    NONE = "NONE"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class ViolationAnalysisRequestAnalysisState(str, Enum):
    APPROVED = "APPROVED"
    NOT_SET = "NOT_SET"
    REJECTED = "REJECTED"

    def __str__(self) -> str:
        return str(self.value)



from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="ForceChangePasswordBody")


@_attrs_define
class ForceChangePasswordBody:
    """
    Attributes:
        username (Union[Unset, str]):
        password (Union[Unset, str]):
        new_password (Union[Unset, str]):
        confirm_password (Union[Unset, str]):
    """

    username: Union[Unset, str] = UNSET
    password: Union[Unset, str] = UNSET
    new_password: Union[Unset, str] = UNSET
    confirm_password: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        username = self.username

        password = self.password

        new_password = self.new_password

        confirm_password = self.confirm_password

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if username is not UNSET:
            field_dict["username"] = username
        if password is not UNSET:
            field_dict["password"] = password
        if new_password is not UNSET:
            field_dict["newPassword"] = new_password
        if confirm_password is not UNSET:
            field_dict["confirmPassword"] = confirm_password

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        username = d.pop("username", UNSET)

        password = d.pop("password", UNSET)

        new_password = d.pop("newPassword", UNSET)

        confirm_password = d.pop("confirmPassword", UNSET)

        force_change_password_body = cls(
            username=username,
            password=password,
            new_password=new_password,
            confirm_password=confirm_password,
        )

        force_change_password_body.additional_properties = d
        return force_change_password_body

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.violation_analysis_analysis_state import ViolationAnalysisAnalysisState
from ..models.violation_analysis_violation_analysis_state import (
    ViolationAnalysisViolationAnalysisState,
)
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.violation_analysis_comment import ViolationAnalysisComment


T = TypeVar("T", bound="ViolationAnalysis")


@_attrs_define
class ViolationAnalysis:
    """
    Attributes:
        analysis_state (ViolationAnalysisAnalysisState):
        analysis_comments (Union[Unset, list['ViolationAnalysisComment']]):
        violation_analysis_state (Union[Unset, ViolationAnalysisViolationAnalysisState]):
        is_suppressed (Union[Unset, bool]):
    """

    analysis_state: ViolationAnalysisAnalysisState
    analysis_comments: Union[Unset, list["ViolationAnalysisComment"]] = UNSET
    violation_analysis_state: Union[Unset, ViolationAnalysisViolationAnalysisState] = (
        UNSET
    )
    is_suppressed: Union[Unset, bool] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        analysis_state = self.analysis_state.value

        analysis_comments: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.analysis_comments, Unset):
            analysis_comments = []
            for analysis_comments_item_data in self.analysis_comments:
                analysis_comments_item = analysis_comments_item_data.to_dict()
                analysis_comments.append(analysis_comments_item)

        violation_analysis_state: Union[Unset, str] = UNSET
        if not isinstance(self.violation_analysis_state, Unset):
            violation_analysis_state = self.violation_analysis_state.value

        is_suppressed = self.is_suppressed

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "analysisState": analysis_state,
            }
        )
        if analysis_comments is not UNSET:
            field_dict["analysisComments"] = analysis_comments
        if violation_analysis_state is not UNSET:
            field_dict["violationAnalysisState"] = violation_analysis_state
        if is_suppressed is not UNSET:
            field_dict["isSuppressed"] = is_suppressed

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.violation_analysis_comment import ViolationAnalysisComment

        d = dict(src_dict)
        analysis_state = ViolationAnalysisAnalysisState(d.pop("analysisState"))

        analysis_comments = []
        _analysis_comments = d.pop("analysisComments", UNSET)
        for analysis_comments_item_data in _analysis_comments or []:
            analysis_comments_item = ViolationAnalysisComment.from_dict(
                analysis_comments_item_data
            )

            analysis_comments.append(analysis_comments_item)

        _violation_analysis_state = d.pop("violationAnalysisState", UNSET)
        violation_analysis_state: Union[Unset, ViolationAnalysisViolationAnalysisState]
        if isinstance(_violation_analysis_state, Unset):
            violation_analysis_state = UNSET
        else:
            violation_analysis_state = ViolationAnalysisViolationAnalysisState(
                _violation_analysis_state
            )

        is_suppressed = d.pop("isSuppressed", UNSET)

        violation_analysis = cls(
            analysis_state=analysis_state,
            analysis_comments=analysis_comments,
            violation_analysis_state=violation_analysis_state,
            is_suppressed=is_suppressed,
        )

        violation_analysis.additional_properties = d
        return violation_analysis

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
    Union,
)
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.repository_type import RepositoryType
from ..types import UNSET, Unset

T = TypeVar("T", bound="Repository")


@_attrs_define
class Repository:
    """
    Attributes:
        type_ (RepositoryType):
        identifier (str):
        url (str):
        resolution_order (int):
        enabled (bool):
        internal (bool):
        uuid (UUID):
        authentication_required (Union[Unset, bool]):
        username (Union[Unset, str]):
        password (Union[Unset, str]):
    """

    type_: RepositoryType
    identifier: str
    url: str
    resolution_order: int
    enabled: bool
    internal: bool
    uuid: UUID
    authentication_required: Union[Unset, bool] = UNSET
    username: Union[Unset, str] = UNSET
    password: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        type_ = self.type_.value

        identifier = self.identifier

        url = self.url

        resolution_order = self.resolution_order

        enabled = self.enabled

        internal = self.internal

        uuid = str(self.uuid)

        authentication_required = self.authentication_required

        username = self.username

        password = self.password

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "type": type_,
                "identifier": identifier,
                "url": url,
                "resolutionOrder": resolution_order,
                "enabled": enabled,
                "internal": internal,
                "uuid": uuid,
            }
        )
        if authentication_required is not UNSET:
            field_dict["authenticationRequired"] = authentication_required
        if username is not UNSET:
            field_dict["username"] = username
        if password is not UNSET:
            field_dict["password"] = password

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        type_ = RepositoryType(d.pop("type"))

        identifier = d.pop("identifier")

        url = d.pop("url")

        resolution_order = d.pop("resolutionOrder")

        enabled = d.pop("enabled")

        internal = d.pop("internal")

        uuid = UUID(d.pop("uuid"))

        authentication_required = d.pop("authenticationRequired", UNSET)

        username = d.pop("username", UNSET)

        password = d.pop("password", UNSET)

        repository = cls(
            type_=type_,
            identifier=identifier,
            url=url,
            resolution_order=resolution_order,
            enabled=enabled,
            internal=internal,
            uuid=uuid,
            authentication_required=authentication_required,
            username=username,
            password=password,
        )

        repository.additional_properties = d
        return repository

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import pytest
from is_empty import empty

import owasp_dt
import test
from owasp_dt.api.license_ import get_license
from owasp_dt.api.policy import create_policy
from owasp_dt.api.policy_condition import create_policy_condition
from owasp_dt.api.violation import get_violations_by_project, get_violations
from owasp_dt.models import Policy, PolicyViolationState, PolicyCondition, PolicyConditionSubject, PolicyConditionOperator, License, PolicyOperator
from owasp_dt.types import UNSET


def test_mit_license(client: owasp_dt.Client):
    def _test_mit_license():
        resp = get_license.sync_detailed(client=client, license_id="MIT")
        assert resp.status_code == 200
        license = resp.parsed
        assert isinstance(license, License)
        test.mit_license_uuid = str(license.uuid)

    test.retry(_test_mit_license, 600)

@pytest.mark.depends(on=['test_mit_license'])
def test_create_test_policy(client: owasp_dt.Client):
    policy = Policy(
        uuid="",
        name="Forbid MIT license",
        violation_state=PolicyViolationState.FAIL,
        operator=PolicyOperator.ANY,
    )
    resp = create_policy.sync_detailed(client=client, body=policy)
    if resp.status_code == 409:
        return
    assert resp.status_code == 201
    policy = resp.parsed
    assert isinstance(policy, Policy)

    condition = PolicyCondition(
        uuid="",
        policy=UNSET,
        subject=PolicyConditionSubject.LICENSE,
        operator=PolicyConditionOperator.IS,
        value=test.mit_license_uuid,
    )
    resp = create_policy_condition.sync_detailed(client=client, uuid=policy.uuid, body=condition)
    assert resp.status_code == 201


@pytest.mark.depends(on=['test_create_test_policy', 'test/test_upload.py::test_upload_sbom'])
def test_get_violations(client: owasp_dt.Client):
    def _get_violations():
        resp = get_violations.sync_detailed(client=client, page_size=1)
        violations = resp.parsed
        assert len(violations) > 0

    test.retry(_get_violations, 600)


@pytest.mark.depends(on=['test/test_projects.py::test_search_project_by_name', 'test/test_upload.py::test_get_scan_status'])
def test_get_project_violations(client: owasp_dt.Client):
    resp = get_violations_by_project.sync_detailed(client=client, uuid=test.project_uuid)
    violations = resp.parsed



from enum import Enum


class AffectedVersionAttributionSource(str, Enum):
    GITHUB = "GITHUB"
    INTERNAL = "INTERNAL"
    NPM = "NPM"
    NVD = "NVD"
    OSSINDEX = "OSSINDEX"
    OSV = "OSV"
    RETIREJS = "RETIREJS"
    SNYK = "SNYK"
    TRIVY = "TRIVY"
    UNKNOWN = "UNKNOWN"
    VULNDB = "VULNDB"

    def __str__(self) -> str:
        return str(self.value)



import pytest

import owasp_dt
import test
from owasp_dt.api.config_property import update_config_property
from owasp_dt.api.metrics import get_vulnerability_metrics
from owasp_dt.api.vulnerability import get_all_vulnerabilities
from owasp_dt.models import ConfigProperty, ConfigPropertyPropertyType


def test_trigger_vulnerabilities_update(client: owasp_dt.Client):
    config_property = ConfigProperty(
        group_name="task-scheduler",
        property_name="nist.mirror.cadence",
        property_value="1",
        property_type=ConfigPropertyPropertyType.NUMBER,
    )
    resp = update_config_property.sync_detailed(client=client, body=config_property)
    assert resp.status_code == 200


def test_enable_nvd(client: owasp_dt.Client):
    config_property = ConfigProperty(
        group_name="vuln-source",
        property_name="nvd.enabled",
        property_value="false",
        property_type=ConfigPropertyPropertyType.BOOLEAN,
    )
    resp = update_config_property.sync_detailed(client=client, body=config_property)
    assert resp.status_code == 200

    config_property.property_value = "true"
    resp = update_config_property.sync_detailed(client=client, body=config_property)
    assert resp.status_code == 200


@pytest.mark.depends(on=['test_trigger_vulnerabilities_update', "test_enable_nvd"])
def test_get_vulnerabilities(client: owasp_dt.Client):
    def _get_vulnerabilities():
        resp = get_all_vulnerabilities.sync_detailed(client=client, page_size=1)
        vulnerabilities = resp.parsed
        assert len(vulnerabilities) > 0

    test.retry(_get_vulnerabilities, 600)


@pytest.mark.depends(on=["test_get_vulnerabilities", 'test/test_upload.py::test_upload_sbom'])
@pytest.mark.xfail(reason="https://github.com/DependencyTrack/dependency-track/issues/5401")
def test_get_vulnerability_metrics(client: owasp_dt.Client):
    def _get_vulnerability_metrics():
        resp = get_vulnerability_metrics.sync_detailed(client=client)
        vulnerabilities = resp.parsed
        assert len(vulnerabilities) > 0

    test.retry(_get_vulnerability_metrics, 10)



import pytest
from tinystream import Opt

import owasp_dt
import test
from owasp_dt.api.project import get_project
from owasp_dt.models import ProjectPropertyPropertyType, ProjectProperty
from test import api


@pytest.mark.depends(on=['test/test_projects.py::test_search_project_by_name'])
def test_upsert_project_property(client: owasp_dt.Client):
    property = ProjectProperty(
        group_name="owasp-dtrack-python-client",
        property_name="test",
        property_type=ProjectPropertyPropertyType.STRING,
        property_value="set",
        description="Custom property test"
    )
    api.upsert_project_property(client=client, uuid=test.project_uuid, property=property)

    def _filter_property(property:ProjectProperty):
        return property.group_name == "owasp-dtrack-python-client" and property.property_name == "test"

    resp = get_project.sync_detailed(client=client, uuid=test.project_uuid)
    project = resp.parsed
    opt_property = Opt(project).map_key("properties").stream().filter(_filter_property).next()
    assert opt_property.present
    assert opt_property.get().property_value == "set"

    property.property_value = "new_value"
    api.upsert_project_property(client=client, uuid=test.project_uuid, property=property)
    resp = get_project.sync_detailed(client=client, uuid=test.project_uuid)
    project = resp.parsed
    opt_property = Opt(project).map_key("properties").stream().filter(_filter_property).next()
    assert opt_property.present
    assert opt_property.get().property_value == "new_value"



from http import HTTPStatus
from typing import Any, Optional, Union, cast

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.vulnerability import Vulnerability
from ...types import Response


def _get_kwargs(
    source: str,
    vuln: str,
) -> dict[str, Any]:
    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/v1/vulnerability/source/{source}/vuln/{vuln}".format(
            source=source,
            vuln=vuln,
        ),
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[Any, Vulnerability]]:
    if response.status_code == 200:
        response_200 = Vulnerability.from_dict(response.json())

        return response_200

    if response.status_code == 401:
        response_401 = cast(Any, None)
        return response_401

    if response.status_code == 404:
        response_404 = cast(Any, None)
        return response_404

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[Any, Vulnerability]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    source: str,
    vuln: str,
    *,
    client: AuthenticatedClient,
) -> Response[Union[Any, Vulnerability]]:
    """Returns a specific vulnerability

     <p>Requires permission <strong>VIEW_PORTFOLIO</strong></p>

    Args:
        source (str):
        vuln (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, Vulnerability]]
    """

    kwargs = _get_kwargs(
        source=source,
        vuln=vuln,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    source: str,
    vuln: str,
    *,
    client: AuthenticatedClient,
) -> Optional[Union[Any, Vulnerability]]:
    """Returns a specific vulnerability

     <p>Requires permission <strong>VIEW_PORTFOLIO</strong></p>

    Args:
        source (str):
        vuln (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, Vulnerability]
    """

    return sync_detailed(
        source=source,
        vuln=vuln,
        client=client,
    ).parsed


async def asyncio_detailed(
    source: str,
    vuln: str,
    *,
    client: AuthenticatedClient,
) -> Response[Union[Any, Vulnerability]]:
    """Returns a specific vulnerability

     <p>Requires permission <strong>VIEW_PORTFOLIO</strong></p>

    Args:
        source (str):
        vuln (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, Vulnerability]]
    """

    kwargs = _get_kwargs(
        source=source,
        vuln=vuln,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    source: str,
    vuln: str,
    *,
    client: AuthenticatedClient,
) -> Optional[Union[Any, Vulnerability]]:
    """Returns a specific vulnerability

     <p>Requires permission <strong>VIEW_PORTFOLIO</strong></p>

    Args:
        source (str):
        vuln (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, Vulnerability]
    """

    return (
        await asyncio_detailed(
            source=source,
            vuln=vuln,
            client=client,
        )
    ).parsed



from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
    Union,
)
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="VulnerabilityAlias")


@_attrs_define
class VulnerabilityAlias:
    """
    Attributes:
        uuid (UUID):
        internal_id (Union[Unset, str]):
        cve_id (Union[Unset, str]):
        ghsa_id (Union[Unset, str]):
        sonatype_id (Union[Unset, str]):
        osv_id (Union[Unset, str]):
        snyk_id (Union[Unset, str]):
        gsd_id (Union[Unset, str]):
        vuln_db_id (Union[Unset, str]):
    """

    uuid: UUID
    internal_id: Union[Unset, str] = UNSET
    cve_id: Union[Unset, str] = UNSET
    ghsa_id: Union[Unset, str] = UNSET
    sonatype_id: Union[Unset, str] = UNSET
    osv_id: Union[Unset, str] = UNSET
    snyk_id: Union[Unset, str] = UNSET
    gsd_id: Union[Unset, str] = UNSET
    vuln_db_id: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        uuid = str(self.uuid)

        internal_id = self.internal_id

        cve_id = self.cve_id

        ghsa_id = self.ghsa_id

        sonatype_id = self.sonatype_id

        osv_id = self.osv_id

        snyk_id = self.snyk_id

        gsd_id = self.gsd_id

        vuln_db_id = self.vuln_db_id

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "uuid": uuid,
            }
        )
        if internal_id is not UNSET:
            field_dict["internalId"] = internal_id
        if cve_id is not UNSET:
            field_dict["cveId"] = cve_id
        if ghsa_id is not UNSET:
            field_dict["ghsaId"] = ghsa_id
        if sonatype_id is not UNSET:
            field_dict["sonatypeId"] = sonatype_id
        if osv_id is not UNSET:
            field_dict["osvId"] = osv_id
        if snyk_id is not UNSET:
            field_dict["snykId"] = snyk_id
        if gsd_id is not UNSET:
            field_dict["gsdId"] = gsd_id
        if vuln_db_id is not UNSET:
            field_dict["vulnDbId"] = vuln_db_id

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        uuid = UUID(d.pop("uuid"))

        internal_id = d.pop("internalId", UNSET)

        cve_id = d.pop("cveId", UNSET)

        ghsa_id = d.pop("ghsaId", UNSET)

        sonatype_id = d.pop("sonatypeId", UNSET)

        osv_id = d.pop("osvId", UNSET)

        snyk_id = d.pop("snykId", UNSET)

        gsd_id = d.pop("gsdId", UNSET)

        vuln_db_id = d.pop("vulnDbId", UNSET)

        vulnerability_alias = cls(
            uuid=uuid,
            internal_id=internal_id,
            cve_id=cve_id,
            ghsa_id=ghsa_id,
            sonatype_id=sonatype_id,
            osv_id=osv_id,
            snyk_id=snyk_id,
            gsd_id=gsd_id,
            vuln_db_id=vuln_db_id,
        )

        vulnerability_alias.additional_properties = d
        return vulnerability_alias

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="ValidateOidcAccessTokenBody")


@_attrs_define
class ValidateOidcAccessTokenBody:
    """
    Attributes:
        id_token (str): An OAuth2 access token
        access_token (Union[Unset, str]):
    """

    id_token: str
    access_token: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        id_token = self.id_token

        access_token = self.access_token

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "idToken": id_token,
            }
        )
        if access_token is not UNSET:
            field_dict["accessToken"] = access_token

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        id_token = d.pop("idToken")

        access_token = d.pop("accessToken", UNSET)

        validate_oidc_access_token_body = cls(
            id_token=id_token,
            access_token=access_token,
        )

        validate_oidc_access_token_body.additional_properties = d
        return validate_oidc_access_token_body

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class GetAllPermissionsResponse200(str, Enum):
    ACCESS_MANAGEMENT = "ACCESS_MANAGEMENT"
    BOM_UPLOAD = "BOM_UPLOAD"
    POLICY_MANAGEMENT = "POLICY_MANAGEMENT"
    POLICY_VIOLATION_ANALYSIS = "POLICY_VIOLATION_ANALYSIS"
    PORTFOLIO_MANAGEMENT = "PORTFOLIO_MANAGEMENT"
    PROJECT_CREATION_UPLOAD = "PROJECT_CREATION_UPLOAD"
    SYSTEM_CONFIGURATION = "SYSTEM_CONFIGURATION"
    TAG_MANAGEMENT = "TAG_MANAGEMENT"
    VIEW_BADGES = "VIEW_BADGES"
    VIEW_POLICY_VIOLATION = "VIEW_POLICY_VIOLATION"
    VIEW_PORTFOLIO = "VIEW_PORTFOLIO"
    VIEW_VULNERABILITY = "VIEW_VULNERABILITY"
    VULNERABILITY_ANALYSIS = "VULNERABILITY_ANALYSIS"
    VULNERABILITY_MANAGEMENT = "VULNERABILITY_MANAGEMENT"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class VulnerabilitySeverity(str, Enum):
    CRITICAL = "CRITICAL"
    HIGH = "HIGH"
    INFO = "INFO"
    LOW = "LOW"
    MEDIUM = "MEDIUM"
    UNASSIGNED = "UNASSIGNED"

    def __str__(self) -> str:
        return str(self.value)



from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
    Union,
)
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="DependencyGraphResponse")


@_attrs_define
class DependencyGraphResponse:
    """
    Attributes:
        uuid (Union[Unset, UUID]):
        name (Union[Unset, str]):
        version (Union[Unset, str]):
        purl (Union[Unset, str]):
        direct_dependencies (Union[Unset, str]):
        latest_version (Union[Unset, str]):
    """

    uuid: Union[Unset, UUID] = UNSET
    name: Union[Unset, str] = UNSET
    version: Union[Unset, str] = UNSET
    purl: Union[Unset, str] = UNSET
    direct_dependencies: Union[Unset, str] = UNSET
    latest_version: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        uuid: Union[Unset, str] = UNSET
        if not isinstance(self.uuid, Unset):
            uuid = str(self.uuid)

        name = self.name

        version = self.version

        purl = self.purl

        direct_dependencies = self.direct_dependencies

        latest_version = self.latest_version

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if uuid is not UNSET:
            field_dict["uuid"] = uuid
        if name is not UNSET:
            field_dict["name"] = name
        if version is not UNSET:
            field_dict["version"] = version
        if purl is not UNSET:
            field_dict["purl"] = purl
        if direct_dependencies is not UNSET:
            field_dict["directDependencies"] = direct_dependencies
        if latest_version is not UNSET:
            field_dict["latestVersion"] = latest_version

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        _uuid = d.pop("uuid", UNSET)
        uuid: Union[Unset, UUID]
        if isinstance(_uuid, Unset):
            uuid = UNSET
        else:
            uuid = UUID(_uuid)

        name = d.pop("name", UNSET)

        version = d.pop("version", UNSET)

        purl = d.pop("purl", UNSET)

        direct_dependencies = d.pop("directDependencies", UNSET)

        latest_version = d.pop("latestVersion", UNSET)

        dependency_graph_response = cls(
            uuid=uuid,
            name=name,
            version=version,
            purl=purl,
            direct_dependencies=direct_dependencies,
            latest_version=latest_version,
        )

        dependency_graph_response.additional_properties = d
        return dependency_graph_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
    Union,
)
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.oidc_group import OidcGroup


T = TypeVar("T", bound="MappedOidcGroup")


@_attrs_define
class MappedOidcGroup:
    """
    Attributes:
        uuid (UUID):
        group (Union[Unset, OidcGroup]):
    """

    uuid: UUID
    group: Union[Unset, "OidcGroup"] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        uuid = str(self.uuid)

        group: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.group, Unset):
            group = self.group.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "uuid": uuid,
            }
        )
        if group is not UNSET:
            field_dict["group"] = group

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.oidc_group import OidcGroup

        d = dict(src_dict)
        uuid = UUID(d.pop("uuid"))

        _group = d.pop("group", UNSET)
        group: Union[Unset, OidcGroup]
        if isinstance(_group, Unset):
            group = UNSET
        else:
            group = OidcGroup.from_dict(_group)

        mapped_oidc_group = cls(
            uuid=uuid,
            group=group,
        )

        mapped_oidc_group.additional_properties = d
        return mapped_oidc_group

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
    Union,
)
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="TaggedCollectionProjectListResponseItem")


@_attrs_define
class TaggedCollectionProjectListResponseItem:
    """
    Attributes:
        uuid (UUID): UUID of the collection project
        name (str): Name of the collection project
        version (Union[Unset, str]): Version of the collection project
    """

    uuid: UUID
    name: str
    version: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        uuid = str(self.uuid)

        name = self.name

        version = self.version

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "uuid": uuid,
                "name": name,
            }
        )
        if version is not UNSET:
            field_dict["version"] = version

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        uuid = UUID(d.pop("uuid"))

        name = d.pop("name")

        version = d.pop("version", UNSET)

        tagged_collection_project_list_response_item = cls(
            uuid=uuid,
            name=name,
            version=version,
        )

        tagged_collection_project_list_response_item.additional_properties = d
        return tagged_collection_project_list_response_item

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class FindingAttribAnalyzerIdentity(str, Enum):
    INTERNAL_ANALYZER = "INTERNAL_ANALYZER"
    NONE = "NONE"
    NPM_AUDIT_ANALYZER = "NPM_AUDIT_ANALYZER"
    OSSINDEX_ANALYZER = "OSSINDEX_ANALYZER"
    SNYK_ANALYZER = "SNYK_ANALYZER"
    TRIVY_ANALYZER = "TRIVY_ANALYZER"
    VULNDB_ANALYZER = "VULNDB_ANALYZER"

    def __str__(self) -> str:
        return str(self.value)



from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
    Union,
    cast,
)
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.project_classifier import ProjectClassifier
from ..models.project_collection_logic import ProjectCollectionLogic
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.external_reference import ExternalReference
    from ..models.organizational_contact import OrganizationalContact
    from ..models.organizational_entity import OrganizationalEntity
    from ..models.project_metadata import ProjectMetadata
    from ..models.project_metrics import ProjectMetrics
    from ..models.project_property import ProjectProperty
    from ..models.project_version import ProjectVersion
    from ..models.tag import Tag
    from ..models.team import Team


T = TypeVar("T", bound="Project")


@_attrs_define
class Project:
    """
    Attributes:
        authors (Union[Unset, list['OrganizationalContact']]):
        publisher (Union[Unset, str]):
        manufacturer (Union[Unset, OrganizationalEntity]):
        supplier (Union[Unset, OrganizationalEntity]):
        group (Union[Unset, str]):
        name (Union[Unset, str]):
        description (Union[Unset, str]):
        version (Union[Unset, str]):
        classifier (Union[Unset, ProjectClassifier]):
        collection_logic (Union[Unset, ProjectCollectionLogic]):
        collection_tag (Union[Unset, Tag]):
        cpe (Union[Unset, str]):
        purl (Union[Unset, str]):
        swid_tag_id (Union[Unset, str]):
        direct_dependencies (Union[Unset, str]):
        uuid (Union[Unset, UUID]):
        parent (Union[Unset, Project]):
        children (Union[None, Unset, list['Project']]):
        properties (Union[Unset, list['ProjectProperty']]):
        tags (Union[Unset, list['Tag']]):
        last_bom_import (Union[Unset, int]): UNIX epoch timestamp in milliseconds
        last_bom_import_format (Union[Unset, str]):
        last_inherited_risk_score (Union[Unset, float]):
        last_vulnerability_analysis (Union[Unset, int]): UNIX epoch timestamp in milliseconds
        active (Union[Unset, bool]):
        is_latest (Union[Unset, bool]):
        access_teams (Union[Unset, list['Team']]):
        external_references (Union[Unset, list['ExternalReference']]):
        metadata (Union[Unset, ProjectMetadata]):
        versions (Union[Unset, list['ProjectVersion']]):
        author (Union[Unset, str]):
        metrics (Union[Unset, ProjectMetrics]):
        bom_ref (Union[Unset, str]):
    """

    authors: Union[Unset, list["OrganizationalContact"]] = UNSET
    publisher: Union[Unset, str] = UNSET
    manufacturer: Union[Unset, "OrganizationalEntity"] = UNSET
    supplier: Union[Unset, "OrganizationalEntity"] = UNSET
    group: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET
    description: Union[Unset, str] = UNSET
    version: Union[Unset, str] = UNSET
    classifier: Union[Unset, ProjectClassifier] = UNSET
    collection_logic: Union[Unset, ProjectCollectionLogic] = UNSET
    collection_tag: Union[Unset, "Tag"] = UNSET
    cpe: Union[Unset, str] = UNSET
    purl: Union[Unset, str] = UNSET
    swid_tag_id: Union[Unset, str] = UNSET
    direct_dependencies: Union[Unset, str] = UNSET
    uuid: Union[Unset, UUID] = UNSET
    parent: Union[Unset, "Project"] = UNSET
    children: Union[None, Unset, list["Project"]] = UNSET
    properties: Union[Unset, list["ProjectProperty"]] = UNSET
    tags: Union[Unset, list["Tag"]] = UNSET
    last_bom_import: Union[Unset, int] = UNSET
    last_bom_import_format: Union[Unset, str] = UNSET
    last_inherited_risk_score: Union[Unset, float] = UNSET
    last_vulnerability_analysis: Union[Unset, int] = UNSET
    active: Union[Unset, bool] = UNSET
    is_latest: Union[Unset, bool] = UNSET
    access_teams: Union[Unset, list["Team"]] = UNSET
    external_references: Union[Unset, list["ExternalReference"]] = UNSET
    metadata: Union[Unset, "ProjectMetadata"] = UNSET
    versions: Union[Unset, list["ProjectVersion"]] = UNSET
    author: Union[Unset, str] = UNSET
    metrics: Union[Unset, "ProjectMetrics"] = UNSET
    bom_ref: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        authors: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.authors, Unset):
            authors = []
            for authors_item_data in self.authors:
                authors_item = authors_item_data.to_dict()
                authors.append(authors_item)

        publisher = self.publisher

        manufacturer: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.manufacturer, Unset):
            manufacturer = self.manufacturer.to_dict()

        supplier: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.supplier, Unset):
            supplier = self.supplier.to_dict()

        group = self.group

        name = self.name

        description = self.description

        version = self.version

        classifier: Union[Unset, str] = UNSET
        if not isinstance(self.classifier, Unset):
            classifier = self.classifier.value

        collection_logic: Union[Unset, str] = UNSET
        if not isinstance(self.collection_logic, Unset):
            collection_logic = self.collection_logic.value

        collection_tag: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.collection_tag, Unset):
            collection_tag = self.collection_tag.to_dict()

        cpe = self.cpe

        purl = self.purl

        swid_tag_id = self.swid_tag_id

        direct_dependencies = self.direct_dependencies

        uuid: Union[Unset, str] = UNSET
        if not isinstance(self.uuid, Unset):
            uuid = str(self.uuid)

        parent: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.parent, Unset):
            parent = self.parent.to_dict()

        children: Union[None, Unset, list[dict[str, Any]]]
        if isinstance(self.children, Unset):
            children = UNSET
        elif isinstance(self.children, list):
            children = []
            for children_type_0_item_data in self.children:
                children_type_0_item = children_type_0_item_data.to_dict()
                children.append(children_type_0_item)

        else:
            children = self.children

        properties: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.properties, Unset):
            properties = []
            for properties_item_data in self.properties:
                properties_item = properties_item_data.to_dict()
                properties.append(properties_item)

        tags: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.tags, Unset):
            tags = []
            for tags_item_data in self.tags:
                tags_item = tags_item_data.to_dict()
                tags.append(tags_item)

        last_bom_import = self.last_bom_import

        last_bom_import_format = self.last_bom_import_format

        last_inherited_risk_score = self.last_inherited_risk_score

        last_vulnerability_analysis = self.last_vulnerability_analysis

        active = self.active

        is_latest = self.is_latest

        access_teams: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.access_teams, Unset):
            access_teams = []
            for access_teams_item_data in self.access_teams:
                access_teams_item = access_teams_item_data.to_dict()
                access_teams.append(access_teams_item)

        external_references: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.external_references, Unset):
            external_references = []
            for external_references_item_data in self.external_references:
                external_references_item = external_references_item_data.to_dict()
                external_references.append(external_references_item)

        metadata: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.metadata, Unset):
            metadata = self.metadata.to_dict()

        versions: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.versions, Unset):
            versions = []
            for versions_item_data in self.versions:
                versions_item = versions_item_data.to_dict()
                versions.append(versions_item)

        author = self.author

        metrics: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.metrics, Unset):
            metrics = self.metrics.to_dict()

        bom_ref = self.bom_ref

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if authors is not UNSET:
            field_dict["authors"] = authors
        if publisher is not UNSET:
            field_dict["publisher"] = publisher
        if manufacturer is not UNSET:
            field_dict["manufacturer"] = manufacturer
        if supplier is not UNSET:
            field_dict["supplier"] = supplier
        if group is not UNSET:
            field_dict["group"] = group
        if name is not UNSET:
            field_dict["name"] = name
        if description is not UNSET:
            field_dict["description"] = description
        if version is not UNSET:
            field_dict["version"] = version
        if classifier is not UNSET:
            field_dict["classifier"] = classifier
        if collection_logic is not UNSET:
            field_dict["collectionLogic"] = collection_logic
        if collection_tag is not UNSET:
            field_dict["collectionTag"] = collection_tag
        if cpe is not UNSET:
            field_dict["cpe"] = cpe
        if purl is not UNSET:
            field_dict["purl"] = purl
        if swid_tag_id is not UNSET:
            field_dict["swidTagId"] = swid_tag_id
        if direct_dependencies is not UNSET:
            field_dict["directDependencies"] = direct_dependencies
        if uuid is not UNSET:
            field_dict["uuid"] = uuid
        if parent is not UNSET:
            field_dict["parent"] = parent
        if children is not UNSET:
            field_dict["children"] = children
        if properties is not UNSET:
            field_dict["properties"] = properties
        if tags is not UNSET:
            field_dict["tags"] = tags
        if last_bom_import is not UNSET:
            field_dict["lastBomImport"] = last_bom_import
        if last_bom_import_format is not UNSET:
            field_dict["lastBomImportFormat"] = last_bom_import_format
        if last_inherited_risk_score is not UNSET:
            field_dict["lastInheritedRiskScore"] = last_inherited_risk_score
        if last_vulnerability_analysis is not UNSET:
            field_dict["lastVulnerabilityAnalysis"] = last_vulnerability_analysis
        if active is not UNSET:
            field_dict["active"] = active
        if is_latest is not UNSET:
            field_dict["isLatest"] = is_latest
        if access_teams is not UNSET:
            field_dict["accessTeams"] = access_teams
        if external_references is not UNSET:
            field_dict["externalReferences"] = external_references
        if metadata is not UNSET:
            field_dict["metadata"] = metadata
        if versions is not UNSET:
            field_dict["versions"] = versions
        if author is not UNSET:
            field_dict["author"] = author
        if metrics is not UNSET:
            field_dict["metrics"] = metrics
        if bom_ref is not UNSET:
            field_dict["bomRef"] = bom_ref

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.external_reference import ExternalReference
        from ..models.organizational_contact import OrganizationalContact
        from ..models.organizational_entity import OrganizationalEntity
        from ..models.project_metadata import ProjectMetadata
        from ..models.project_metrics import ProjectMetrics
        from ..models.project_property import ProjectProperty
        from ..models.project_version import ProjectVersion
        from ..models.tag import Tag
        from ..models.team import Team

        d = dict(src_dict)
        authors = []
        _authors = d.pop("authors", UNSET)
        for authors_item_data in _authors or []:
            authors_item = OrganizationalContact.from_dict(authors_item_data)

            authors.append(authors_item)

        publisher = d.pop("publisher", UNSET)

        _manufacturer = d.pop("manufacturer", UNSET)
        manufacturer: Union[Unset, OrganizationalEntity]
        if isinstance(_manufacturer, Unset):
            manufacturer = UNSET
        else:
            manufacturer = OrganizationalEntity.from_dict(_manufacturer)

        _supplier = d.pop("supplier", UNSET)
        supplier: Union[Unset, OrganizationalEntity]
        if isinstance(_supplier, Unset):
            supplier = UNSET
        else:
            supplier = OrganizationalEntity.from_dict(_supplier)

        group = d.pop("group", UNSET)

        name = d.pop("name", UNSET)

        description = d.pop("description", UNSET)

        version = d.pop("version", UNSET)

        _classifier = d.pop("classifier", UNSET)
        classifier: Union[Unset, ProjectClassifier]
        if isinstance(_classifier, Unset):
            classifier = UNSET
        else:
            classifier = ProjectClassifier(_classifier)

        _collection_logic = d.pop("collectionLogic", UNSET)
        collection_logic: Union[Unset, ProjectCollectionLogic]
        if isinstance(_collection_logic, Unset):
            collection_logic = UNSET
        else:
            collection_logic = ProjectCollectionLogic(_collection_logic)

        _collection_tag = d.pop("collectionTag", UNSET)
        collection_tag: Union[Unset, Tag]
        if isinstance(_collection_tag, Unset):
            collection_tag = UNSET
        else:
            collection_tag = Tag.from_dict(_collection_tag)

        cpe = d.pop("cpe", UNSET)

        purl = d.pop("purl", UNSET)

        swid_tag_id = d.pop("swidTagId", UNSET)

        direct_dependencies = d.pop("directDependencies", UNSET)

        _uuid = d.pop("uuid", UNSET)
        uuid: Union[Unset, UUID]
        if isinstance(_uuid, Unset):
            uuid = UNSET
        else:
            uuid = UUID(_uuid)

        _parent = d.pop("parent", UNSET)
        parent: Union[Unset, Project]
        if isinstance(_parent, Unset):
            parent = UNSET
        else:
            parent = Project.from_dict(_parent)

        def _parse_children(data: object) -> Union[None, Unset, list["Project"]]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                children_type_0 = []
                _children_type_0 = data
                for children_type_0_item_data in _children_type_0:
                    children_type_0_item = Project.from_dict(children_type_0_item_data)

                    children_type_0.append(children_type_0_item)

                return children_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, list["Project"]], data)

        children = _parse_children(d.pop("children", UNSET))

        properties = []
        _properties = d.pop("properties", UNSET)
        for properties_item_data in _properties or []:
            properties_item = ProjectProperty.from_dict(properties_item_data)

            properties.append(properties_item)

        tags = []
        _tags = d.pop("tags", UNSET)
        for tags_item_data in _tags or []:
            tags_item = Tag.from_dict(tags_item_data)

            tags.append(tags_item)

        last_bom_import = d.pop("lastBomImport", UNSET)

        last_bom_import_format = d.pop("lastBomImportFormat", UNSET)

        last_inherited_risk_score = d.pop("lastInheritedRiskScore", UNSET)

        last_vulnerability_analysis = d.pop("lastVulnerabilityAnalysis", UNSET)

        active = d.pop("active", UNSET)

        is_latest = d.pop("isLatest", UNSET)

        access_teams = []
        _access_teams = d.pop("accessTeams", UNSET)
        for access_teams_item_data in _access_teams or []:
            access_teams_item = Team.from_dict(access_teams_item_data)

            access_teams.append(access_teams_item)

        external_references = []
        _external_references = d.pop("externalReferences", UNSET)
        for external_references_item_data in _external_references or []:
            external_references_item = ExternalReference.from_dict(
                external_references_item_data
            )

            external_references.append(external_references_item)

        _metadata = d.pop("metadata", UNSET)
        metadata: Union[Unset, ProjectMetadata]
        if isinstance(_metadata, Unset):
            metadata = UNSET
        else:
            metadata = ProjectMetadata.from_dict(_metadata)

        versions = []
        _versions = d.pop("versions", UNSET)
        for versions_item_data in _versions or []:
            versions_item = ProjectVersion.from_dict(versions_item_data)

            versions.append(versions_item)

        author = d.pop("author", UNSET)

        _metrics = d.pop("metrics", UNSET)
        metrics: Union[Unset, ProjectMetrics]
        if isinstance(_metrics, Unset):
            metrics = UNSET
        else:
            metrics = ProjectMetrics.from_dict(_metrics)

        bom_ref = d.pop("bomRef", UNSET)

        project = cls(
            authors=authors,
            publisher=publisher,
            manufacturer=manufacturer,
            supplier=supplier,
            group=group,
            name=name,
            description=description,
            version=version,
            classifier=classifier,
            collection_logic=collection_logic,
            collection_tag=collection_tag,
            cpe=cpe,
            purl=purl,
            swid_tag_id=swid_tag_id,
            direct_dependencies=direct_dependencies,
            uuid=uuid,
            parent=parent,
            children=children,
            properties=properties,
            tags=tags,
            last_bom_import=last_bom_import,
            last_bom_import_format=last_bom_import_format,
            last_inherited_risk_score=last_inherited_risk_score,
            last_vulnerability_analysis=last_vulnerability_analysis,
            active=active,
            is_latest=is_latest,
            access_teams=access_teams,
            external_references=external_references,
            metadata=metadata,
            versions=versions,
            author=author,
            metrics=metrics,
            bom_ref=bom_ref,
        )

        project.additional_properties = d
        return project

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
    Union,
)
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.notification_rule_notification_level import (
    NotificationRuleNotificationLevel,
)
from ..models.notification_rule_notify_on_item import NotificationRuleNotifyOnItem
from ..models.notification_rule_scope import NotificationRuleScope
from ..models.notification_rule_trigger_type import NotificationRuleTriggerType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.notification_publisher import NotificationPublisher
    from ..models.project import Project
    from ..models.tag import Tag
    from ..models.team import Team


T = TypeVar("T", bound="NotificationRule")


@_attrs_define
class NotificationRule:
    """
    Attributes:
        name (str):
        scope (NotificationRuleScope):
        trigger_type (NotificationRuleTriggerType):
        uuid (UUID):
        enabled (Union[Unset, bool]):
        notify_children (Union[Unset, bool]):
        log_successful_publish (Union[Unset, bool]):
        notification_level (Union[Unset, NotificationRuleNotificationLevel]):
        projects (Union[Unset, list['Project']]):
        tags (Union[Unset, list['Tag']]):
        teams (Union[Unset, list['Team']]):
        notify_on (Union[Unset, list[NotificationRuleNotifyOnItem]]):
        message (Union[Unset, str]):
        publisher (Union[Unset, NotificationPublisher]):
        publisher_config (Union[Unset, str]):
        schedule_last_triggered_at (Union[Unset, int]): When the schedule last triggered, as UNIX epoch timestamp in
            milliseconds
        schedule_next_trigger_at (Union[Unset, int]): When the schedule triggers next, as UNIX epoch timestamp in
            milliseconds
        schedule_cron (Union[Unset, str]): Schedule of this rule as cron expression. Must not be set for rules with
            trigger type EVENT.
        schedule_skip_unchanged (Union[Unset, bool]): Whether to skip emitting a scheduled notification if it doesn't
            contain any changes since its last emission. Must not be set for rules with trigger type EVENT.
    """

    name: str
    scope: NotificationRuleScope
    trigger_type: NotificationRuleTriggerType
    uuid: UUID
    enabled: Union[Unset, bool] = UNSET
    notify_children: Union[Unset, bool] = UNSET
    log_successful_publish: Union[Unset, bool] = UNSET
    notification_level: Union[Unset, NotificationRuleNotificationLevel] = UNSET
    projects: Union[Unset, list["Project"]] = UNSET
    tags: Union[Unset, list["Tag"]] = UNSET
    teams: Union[Unset, list["Team"]] = UNSET
    notify_on: Union[Unset, list[NotificationRuleNotifyOnItem]] = UNSET
    message: Union[Unset, str] = UNSET
    publisher: Union[Unset, "NotificationPublisher"] = UNSET
    publisher_config: Union[Unset, str] = UNSET
    schedule_last_triggered_at: Union[Unset, int] = UNSET
    schedule_next_trigger_at: Union[Unset, int] = UNSET
    schedule_cron: Union[Unset, str] = UNSET
    schedule_skip_unchanged: Union[Unset, bool] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        name = self.name

        scope = self.scope.value

        trigger_type = self.trigger_type.value

        uuid = str(self.uuid)

        enabled = self.enabled

        notify_children = self.notify_children

        log_successful_publish = self.log_successful_publish

        notification_level: Union[Unset, str] = UNSET
        if not isinstance(self.notification_level, Unset):
            notification_level = self.notification_level.value

        projects: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.projects, Unset):
            projects = []
            for projects_item_data in self.projects:
                projects_item = projects_item_data.to_dict()
                projects.append(projects_item)

        tags: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.tags, Unset):
            tags = []
            for tags_item_data in self.tags:
                tags_item = tags_item_data.to_dict()
                tags.append(tags_item)

        teams: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.teams, Unset):
            teams = []
            for teams_item_data in self.teams:
                teams_item = teams_item_data.to_dict()
                teams.append(teams_item)

        notify_on: Union[Unset, list[str]] = UNSET
        if not isinstance(self.notify_on, Unset):
            notify_on = []
            for notify_on_item_data in self.notify_on:
                notify_on_item = notify_on_item_data.value
                notify_on.append(notify_on_item)

        message = self.message

        publisher: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.publisher, Unset):
            publisher = self.publisher.to_dict()

        publisher_config = self.publisher_config

        schedule_last_triggered_at = self.schedule_last_triggered_at

        schedule_next_trigger_at = self.schedule_next_trigger_at

        schedule_cron = self.schedule_cron

        schedule_skip_unchanged = self.schedule_skip_unchanged

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "scope": scope,
                "triggerType": trigger_type,
                "uuid": uuid,
            }
        )
        if enabled is not UNSET:
            field_dict["enabled"] = enabled
        if notify_children is not UNSET:
            field_dict["notifyChildren"] = notify_children
        if log_successful_publish is not UNSET:
            field_dict["logSuccessfulPublish"] = log_successful_publish
        if notification_level is not UNSET:
            field_dict["notificationLevel"] = notification_level
        if projects is not UNSET:
            field_dict["projects"] = projects
        if tags is not UNSET:
            field_dict["tags"] = tags
        if teams is not UNSET:
            field_dict["teams"] = teams
        if notify_on is not UNSET:
            field_dict["notifyOn"] = notify_on
        if message is not UNSET:
            field_dict["message"] = message
        if publisher is not UNSET:
            field_dict["publisher"] = publisher
        if publisher_config is not UNSET:
            field_dict["publisherConfig"] = publisher_config
        if schedule_last_triggered_at is not UNSET:
            field_dict["scheduleLastTriggeredAt"] = schedule_last_triggered_at
        if schedule_next_trigger_at is not UNSET:
            field_dict["scheduleNextTriggerAt"] = schedule_next_trigger_at
        if schedule_cron is not UNSET:
            field_dict["scheduleCron"] = schedule_cron
        if schedule_skip_unchanged is not UNSET:
            field_dict["scheduleSkipUnchanged"] = schedule_skip_unchanged

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.notification_publisher import NotificationPublisher
        from ..models.project import Project
        from ..models.tag import Tag
        from ..models.team import Team

        d = dict(src_dict)
        name = d.pop("name")

        scope = NotificationRuleScope(d.pop("scope"))

        trigger_type = NotificationRuleTriggerType(d.pop("triggerType"))

        uuid = UUID(d.pop("uuid"))

        enabled = d.pop("enabled", UNSET)

        notify_children = d.pop("notifyChildren", UNSET)

        log_successful_publish = d.pop("logSuccessfulPublish", UNSET)

        _notification_level = d.pop("notificationLevel", UNSET)
        notification_level: Union[Unset, NotificationRuleNotificationLevel]
        if isinstance(_notification_level, Unset):
            notification_level = UNSET
        else:
            notification_level = NotificationRuleNotificationLevel(_notification_level)

        projects = []
        _projects = d.pop("projects", UNSET)
        for projects_item_data in _projects or []:
            projects_item = Project.from_dict(projects_item_data)

            projects.append(projects_item)

        tags = []
        _tags = d.pop("tags", UNSET)
        for tags_item_data in _tags or []:
            tags_item = Tag.from_dict(tags_item_data)

            tags.append(tags_item)

        teams = []
        _teams = d.pop("teams", UNSET)
        for teams_item_data in _teams or []:
            teams_item = Team.from_dict(teams_item_data)

            teams.append(teams_item)

        notify_on = []
        _notify_on = d.pop("notifyOn", UNSET)
        for notify_on_item_data in _notify_on or []:
            notify_on_item = NotificationRuleNotifyOnItem(notify_on_item_data)

            notify_on.append(notify_on_item)

        message = d.pop("message", UNSET)

        _publisher = d.pop("publisher", UNSET)
        publisher: Union[Unset, NotificationPublisher]
        if isinstance(_publisher, Unset):
            publisher = UNSET
        else:
            publisher = NotificationPublisher.from_dict(_publisher)

        publisher_config = d.pop("publisherConfig", UNSET)

        schedule_last_triggered_at = d.pop("scheduleLastTriggeredAt", UNSET)

        schedule_next_trigger_at = d.pop("scheduleNextTriggerAt", UNSET)

        schedule_cron = d.pop("scheduleCron", UNSET)

        schedule_skip_unchanged = d.pop("scheduleSkipUnchanged", UNSET)

        notification_rule = cls(
            name=name,
            scope=scope,
            trigger_type=trigger_type,
            uuid=uuid,
            enabled=enabled,
            notify_children=notify_children,
            log_successful_publish=log_successful_publish,
            notification_level=notification_level,
            projects=projects,
            tags=tags,
            teams=teams,
            notify_on=notify_on,
            message=message,
            publisher=publisher,
            publisher_config=publisher_config,
            schedule_last_triggered_at=schedule_last_triggered_at,
            schedule_next_trigger_at=schedule_next_trigger_at,
            schedule_cron=schedule_cron,
            schedule_skip_unchanged=schedule_skip_unchanged,
        )

        notification_rule.additional_properties = d
        return notification_rule

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class PolicyConditionSubject(str, Enum):
    AGE = "AGE"
    COMPONENT_HASH = "COMPONENT_HASH"
    COORDINATES = "COORDINATES"
    CPE = "CPE"
    CWE = "CWE"
    EPSS = "EPSS"
    LICENSE = "LICENSE"
    LICENSE_GROUP = "LICENSE_GROUP"
    PACKAGE_URL = "PACKAGE_URL"
    SEVERITY = "SEVERITY"
    SWID_TAGID = "SWID_TAGID"
    VERSION = "VERSION"
    VERSION_DISTANCE = "VERSION_DISTANCE"
    VULNERABILITY_ID = "VULNERABILITY_ID"

    def __str__(self) -> str:
        return str(self.value)
